@@ -13,6 +13,7 @@ from hashlib import md5
 from werkzeug.http import parse_authorization_header
 
 from flask import request, make_response
+from six.moves.urllib.parse import urlparse, urlunparse
 
 
 from .structures import CaseInsensitiveDict
@@ -76,6 +77,7 @@ def json_safe(string, content_type='application/octet-stream'):
     URL scheme was chosen for its simplicity.
     """
     try:
+        string = string.decode('utf-8')
         _encoded = json.dumps(string)
         return string
     except (ValueError, TypeError):
@@ -133,6 +135,17 @@ def semiflatten(multi):
     else:
         return multi
 
+def get_url(request):
+    """
+    Since we might be hosted behind a proxy, we need to check the
+    X-Forwarded-Proto header to find out what protocol was used to access us.
+    """
+    if 'X-Forwarded-Proto' not in request.headers:
+        return request.url
+    url = list(urlparse(request.url))
+    url[0] = request.headers.get('X-Forwarded-Proto')
+    return urlunparse(url)
+
 
 def get_dict(*keys, **extras):
     """Returns request dict of given keys."""
@@ -140,7 +153,6 @@ def get_dict(*keys, **extras):
     _keys = ('url', 'args', 'form', 'data', 'origin', 'headers', 'files', 'json')
 
     assert all(map(_keys.__contains__, keys))
-
     data = request.data
     form = request.form
     form = semiflatten(request.form)
@@ -151,7 +163,7 @@ def get_dict(*keys, **extras):
         _json = None
 
     d = dict(
-        url=request.url,
+        url=get_url(request),
         args=semiflatten(request.args),
         form=form,
         data=json_safe(data),
